/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_BINARY_DYNAMIC_VANTAGE_POINT_VISITOR__
#define __DD_BINARY_DYNAMIC_VANTAGE_POINT_VISITOR__

#include "DDBinaryVantagePointVisitor.h"

namespace double_description {

    class BinaryDynamicVantagePointVisitor : public BinaryVantagePointVisitor {

        private:
        const bool mUniqueVP;

        protected:
        bool unique_vp () const {
            return mUniqueVP;
        }

        bool use_vp_for_verification () const {
            return true;
        }

        public:
        BinaryDynamicVantagePointVisitor ( const unsigned short depth,
                                           const unsigned short capacity,
                                           const bool unique_vp = false ) :
            BinaryVantagePointVisitor( depth, capacity ),
            mUniqueVP( unique_vp ) {}

        void before ( const RayReferenceContainer & container ) {
            BinaryVantagePointVisitor::before( container );

            mVantagePoint = container.first().activeSet; /* Not the best solution! */
        }

        void onFinish () {
            BinaryVantagePointVisitor::onFinish();
            if ( !mSuperDistanceRange.empty() && mSuperDistanceRange.ub() == mSuperDistanceRange.lb() + 1 ) {
                /* change the vantage point for the next round */
                set( BINARY_VP_SUPERSET ).rotate();
            }
        }
    };
}

#endif // __DD__BINARY_DYNAMIC_VANTAGE_POINT_VISITOR__
