/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_BINARY_VANTAGE_POINT_VISITOR__
#define __DD_BINARY_VANTAGE_POINT_VISITOR__

#include "DDVantagePointVisitor.h"

#define BINARY_VP_SUBSET        0
#define BINARY_VP_SUPERSET      1

namespace double_description {

    class BinaryVantagePointVisitor : public VantagePointVisitor {

        protected:
        double mAverage;

        void onInit ( const RayReference & ref ) {
            const unsigned short distance = dist( ref.activeSet );
            mAverage = ( mAverage * mCurrentEntry + distance )/( mCurrentEntry + 1 );
        }

        bool subset_condition ( const RayReference & ref ) const {
            return dist( ref.activeSet ) < mAverage;
        }

        unsigned short dist ( const BitList & element ) const {
            return mVantagePoint.getCommonPopCnt( element );
        }

        virtual bool unique_vp () const {
            return false;
        }

        virtual bool use_vp_for_verification () const = 0;

        void onSave ( const RayReference & ref ) {

            if ( !unique_vp () || likely( ref.activeSet != mVantagePoint ) ) {
                if ( subset_condition( ref ) ) {
                    set( BINARY_VP_SUBSET ).insert( ref );
                    mSubDistanceRange.update( mVantagePoint.getCommonPopCnt( ref.activeSet ) );
                    mSubPopCntRange.update( ref.degeneracy );
                } else {
                    set( BINARY_VP_SUPERSET ).insert( ref );
                    mSuperDistanceRange.update( mVantagePoint.getCommonPopCnt( ref.activeSet ) );
                    mSuperPopCntRange.update( ref.degeneracy );
                }        
            }    
        }

        public:
        BinaryVantagePointVisitor ( const unsigned short depth,
                                    const unsigned short capacity ) :
            VantagePointVisitor( depth, capacity, 2 ),
            mAverage( 0.0 ) {}

        BinaryVantagePointVisitor ( const unsigned short depth,
                                    const unsigned short capacity,
                                    const BitList & point ) :
            VantagePointVisitor( depth, capacity, 2, point ),
            mAverage( 0.0 ) {}

        virtual void before ( const RayReferenceContainer & container ) {
            VantagePointVisitor::before( container );
        
            mInitRound = true;
            mAverage = 0.0;
        }

        const PreliminaryCondition * createNarrowingCondition ( const unsigned short index,
                                                                const unsigned short bits ) const {

            if ( index == BINARY_VP_SUBSET ) {
                return new VantagePointPreliminaryCondition( set( BINARY_VP_SUBSET ).union_map(), bits, mVantagePoint, mSubDistanceRange, mSubPopCntRange, use_vp_for_verification() );
            } else {
                return new VantagePointPreliminaryCondition( set( BINARY_VP_SUPERSET ).union_map(), bits, mVantagePoint, mSuperDistanceRange, mSuperPopCntRange, use_vp_for_verification() );
            }
        }

    #ifdef __USE_TEMPLATES__
        void refineCondition ( const unsigned int index,
                               const unsigned short bits,
                               VantagePointPreliminaryCondition & cond ) const {
            BitSplitVisitor::refineCondition( index, bits, cond );

            if ( index == BINARY_VP_SUBSET ) {
                cond.init( mVantagePoint, mSubDistanceRange, mSubPopCntRange, use_vp_for_verification() );
            } else {
                cond.init( mVantagePoint, mSuperDistanceRange, mSuperPopCntRange, use_vp_for_verification() );
            } 
        }

        bool supportsExtendedCondition () {
            return true;    
        }
    #endif
    };
}

#endif // __DD_BINARY_VANTAGE_POINT_VISITOR__
