/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_COMMON_BIT_STRATEGY__
#define __DD_COMMON_BIT_STRATEGY__

#include <vector>

#include "DDBitSelectStrategy.h"

namespace double_description {

    class CommonBitStrategy : public BitSelectStrategy {
        
        public:
        CommonBitStrategy ( const RayReferenceContainer & entries,
                            const unsigned int plane ) :
            BitSelectStrategy( entries, plane ) {}     

        CommonBitStrategy ( const std::deque<EdgeCandidate> & entries,
                            const unsigned int plane ) :
            BitSelectStrategy( entries, plane ) {}     

        std::vector<unsigned int> createMapping () const {

            std::vector<unsigned int> mapping;
            const std::vector<unsigned long> & population = getPopulationStats();
            const unsigned long target = getTargetValue( population );

            for ( unsigned int index = 0; index < mPlane; index++ ) {
 
                unsigned int position = 0;

                for ( unsigned int prev = 0; prev < index; prev++ ) {
                    if ( isOrderingCorrect( population[prev], population[index], target ) ) {
                        position++;
                    }
                }

                mapping.insert( mapping.begin() + position, index );
            }

            return mapping;
        }

        bool isOrderingCorrect ( unsigned long first, unsigned long second, const unsigned long target ) const {
            unsigned long first_diff = first > target ? first - target : target - first;
            unsigned long second_diff = second > target ? second - target : target - second;

            return ( first_diff < second_diff && closestToTarget() ) || ( first_diff >= second_diff && !closestToTarget() );
        }

        virtual unsigned long getTargetValue( const std::vector<unsigned long> & population ) const = 0;
        
        virtual bool closestToTarget () const {
            return true;
        } 
    };
}

#endif // __DD_COMMON_BIT_STRATEGY&
