/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_PRELIMINARY_CONDITION__
#define __DD_PRELIMINARY_CONDITION__

#include "DDBitList.h"
#include "DDMapCondition.h"
#include "DDVerificationCondition.h"

namespace double_description {

	class PreliminaryCondition : virtual public MapCondition {

		private:
		unsigned int mMinCommonBits;

		public:
        static const PreliminaryCondition TRIVIAL_FAIL;

#ifdef __STATS_OUTPUT__
        static unsigned long NARROWING_OPS;
        static unsigned long VERIFICATION_OPS;

        static void start_new_stat_session () {
            NARROWING_OPS = 0;
            VERIFICATION_OPS = 0;
        }
#endif

        PreliminaryCondition () :
            MapCondition(),
			mMinCommonBits( 1 ) {}            

    #ifdef __USE_TEMPLATES__
        void set ( const BitList & map ) {
            mMap = map;
        }

        void set ( const unsigned short bits ) {
            mMinCommonBits = bits;
        }
    #endif

		PreliminaryCondition ( const BitList & map, const unsigned int minCommonBits ) :
			MapCondition( map ),
			mMinCommonBits( minCommonBits ) {}

		virtual ~PreliminaryCondition() {}

		virtual bool holdsOnFace ( const BitList & activeSetUnion ) const {
			return mMap.getCommonPopCnt( activeSetUnion ) >= mMinCommonBits;
		}

		virtual bool holdsOnFace( const BitList & activeSetUnion, const BitList &, const BitList & ) const { 
			return holdsOnFace( activeSetUnion );
		}

        bool holdsOnRay ( const BitList & activeSet ) const {
			return mMap.getCommonPopCnt( activeSet ) >= mMinCommonBits;
		}

		long getMinCommonBits () const {
			return mMinCommonBits;
		}

    #ifndef __USE_TEMPLATES__
        virtual
    #endif
        bool proceed_verification ( const BitList & set ) const {
    #ifdef __STATS_OUTPUT__
            PreliminaryCondition::VERIFICATION_OPS++;
    #endif
            return mMap.contains( set );
        }

    #ifndef __USE_TEMPLATES__
        virtual
    #endif
        bool proceed_narrowing( const BitList & set ) const {
    #ifdef __STATS_OUTPUT__
            PreliminaryCondition::NARROWING_OPS++;
    #endif
            return mMap.getCommonPopCnt( set ) >= mMinCommonBits;
        }

        bool feasable( const BitList & set_1 , const BitList & set_2 ) const {
    #ifdef __STATS_OUTPUT__
            PreliminaryCondition::NARROWING_OPS++;
    #endif
            return set_1.getCommonPopCnt( set_2 ) >= mMinCommonBits;
        }

    #ifndef __USE_TEMPLATES__
        virtual
    #endif
        bool isRedundant ( const VerificationCondition & ) const {
            return false;
        }

        virtual const std::string toString () const;
	};
}

#endif // __DD_PRELIMINARY_CONDITION__
