/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_PRELIMINARY_TRAVERSE_VISITOR__
#define __DD_PRELIMINARY_TRAVERSE_VISITOR__

#include <deque>
#include <vector>

#include "DDTreeNode.h"
#include "DDRayRefVisitor.h"

namespace double_description {

    class PreliminaryTraverseVisitor : public RayRefVisitor {

        private:
        const unsigned short mMinCommonBits;

        protected:
        const TreeNode & mTree;
        std::deque<struct EdgeCandidate> mCandidates;

        bool hasFinished () const {
            return true;
        }

        public:
        PreliminaryTraverseVisitor( const TreeNode & tree,
                                    const unsigned short bits ) :
            RayRefVisitor(),
            mMinCommonBits( bits ),
            mTree( tree ) {}

        void onStart () {
            RayRefVisitor::onStart();

            mCandidates.clear();
        }

        virtual void onVisit ( const struct RayReference & current ) {
            RayRefVisitor::onVisit( current );

            mTree.preliminaryTraverse( current, mCandidates );
        }

        const std::deque<struct EdgeCandidate> & candidates () const {
            return mCandidates;
        }   

    };
}

#endif // __DD_PRELIMINARY_TRAVERSE_VISITOR__
