/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <sys/time.h>

#include "DDSingleSetRayGenerator.h"

#ifdef __ALGEBRAIC_TEST__
    #include "DDSingleSetFace.h"
#endif

#include "DDStats.h"
#include "DDOptions.h"
#include "DDLogging.h"

using namespace double_description;

#ifdef __MULTITHREADING__
SingleSetRayGenerator::SingleSetRayGenerator ( const Matrix& system, const Matrix& transformator, const unsigned short threads ) :
#else
#ifdef __COMPRESSION__
SingleSetRayGenerator::SingleSetRayGenerator ( const Matrix& original, const Matrix& system, const Matrix& transformator ) :
	mOriginal( original ),
#else
SingleSetRayGenerator::SingleSetRayGenerator ( const Matrix& system, const Matrix& transformator ) :
#endif // __COMPRESSION__
#endif // __MULTITHREADING__
	mSystem( system ),
	mCache( system, transformator ),
	mRankCalculator( system.getRows(), system.getColumns() ) {
//	mGraph( 14, 6 ) { @ToDo Check if need this thing at all.

#ifndef __MULTITHREADING__
	mCandidatesBuffer.reserve( __CANDIDATES_BUFFER_SIZE__ );
#endif

}

void SingleSetRayGenerator::calculateBase () {

/*	Matrix base = mSystem.getSubMatrix( 0, mSystem.getColumns() );

	for ( unsigned long row = 1; row < base.getRows(); row++ ) {
		for ( unsigned long column = 0; column < base.getColumns() - row; column++ ) {

			if ( base.isNonZeroAt( base.getRows() - row - 1, column ) ) {
				base.setValue( base.getRows() - row - 1, column, base[base.getRows() - row - 1][column] * base[base.getRows() - row][base.getRows() - row] );
			}
		}
	} */

    ExtremeRayUncompressor decompressor( mSystem.getRows(), mSystem.getColumns(), BitList::getBlocks( mSystem.getRows() ));

	for ( unsigned long row = 0; row < mSystem.getColumns(); row++ ) {

		BitList surface_map( BitList::getBlocks( mSystem.getRows() ) );
		RepresentationVector vector( mSystem.getColumns() );
		SingleSetRay* ray;
		mpz_class* coeffs = new mpz_class[ mSystem.getRows() - mSystem.getColumns() ];

        surface_map.setBits( 0, mSystem.getColumns() );
		surface_map.unsetBit( row );

	/*	vector.setValue( row, base[row][row] );

		for ( unsigned long i = row + 1; i < base.getRows(); i++ ) {
			for ( unsigned long column = 0; column < i; column++ ) {
				if ( vector.isNonZeroAt( column ) && base.isNonZeroAt( i, column ) ) {
					vector.add( i, vector.getValue( column ) * base[i][column] );
				}
			}
			vector.divide( i, base[i][i] );
			vector.negate( i );
		}

		vector.simplify(); */

        decompressor.calcExtremeRay( mSystem, surface_map, vector );

		if ( vector * mSystem[row] < 0 ) {
			vector.negate();
		}

		unsigned long index;

		for ( index = mSystem.getColumns(); index < mSystem.getRows(); index++ ) {
			vector.calcScalarProduct( mSystem[index], coeffs[ index - mSystem.getColumns() ] );
			if ( coeffs[ index - mSystem.getColumns() ] == 0 ) {
				surface_map.setBit( index );
			} else if ( coeffs[ index - mSystem.getColumns() ] < 0 ) {
				break;
			}
		}

		for ( ; index < mSystem.getRows(); index++ ) {
			vector.calcScalarProduct( mSystem[index], coeffs[ index - mSystem.getColumns() ] );
		}
		
		ray = new SingleSetRay( vector, coeffs, surface_map );

		mCache.insert( ray );
	}
}

void SingleSetRayGenerator::calculateStep ( const unsigned long plane ) {

#ifdef __BIT_PATTERN_TREES__
	BitPatternTreeCacheEntry<SingleSetRay> & cache_entry =  mCache.next();// */mCache.getEntry( plane - mSystem.getColumns() );
#else
	#ifdef __POPULATION_TREES__
	PopulationTreeCacheEntry<SingleSetRay> & cache_entry = mCache.getEntry( plane - mSystem.getColumns() );
	#else
	RayCacheEntry<SingleSetRay>& cache_entry = mCache.getEntry( plane - mSystem.getColumns() );
	#endif // __POPULATION_TREES__
#endif // __BIT_PATTERN_TREES__

/*	const unsigned long blocks = BitList::getBlocks( mSystem.getRows() ); */
	const unsigned long min_common_bits = mSystem.getColumns() - 2;

/*	const struct ActiveSetContainer<SingleSetRay> p_container = cache_entry.getPositiveContainer();
	const struct ActiveSetContainer<SingleSetRay> n_container = cache_entry.getNegativeContainer(); */

#ifdef __ALGEBRAIC_TEST__
	std::vector<const SingleSetFace*> proven_faces; 
#endif

/*	const unsigned long adj_candidates = p_container.ray_count * n_container.ray_count; */

	DD_CONCAT ( "[" << plane - mSystem.getColumns() + 1 << "/" << mSystem.getRows() - mSystem.getColumns() << "] "
                    << "( pos. (deg.)/neg. (deg.)/elim. compl./ver. cand./acc./ver. lg(n)^2/ver. n/extr. rays) = " )

	START_TIMER( STEP_TIME )

#ifdef __HYBRID_TREES__
    const struct StepStats stats = cache_entry.fillCandidatesBuffer( mCandidatesBuffer, min_common_bits ); 
#else
	const struct StepStats stats = cache_entry.fillCandidatesBuffer( mCandidatesBuffer, min_common_bits ); 
#endif

    DD_CONCAT( cache_entry.getPositiveCount() << "(" )
    if ( stats.degeneracyPos != ULONG_MAX ) {
        DD_CONCAT ( stats.degeneracyPos )
    } else {
        DD_CONCAT ( "-" )
    }

    DD_CONCAT( ")/" << cache_entry.getNegativeCount() << "(" )

    if ( stats.degeneracyNeg != ULONG_MAX ) {
        DD_CONCAT ( stats.degeneracyNeg )
    } else {
        DD_CONCAT ( "-" )
    }

	DD_CONCAT ( ")/" << stats.elimComplexity << "/" << stats.verificationCandidates << "/" << stats.verified << "/" )

    if ( stats.verComplexityLg != 0.0 ) {
        DD_CONCAT ( stats.verComplexityLg )
    } else {
        DD_CONCAT ( "-" )
    }

    DD_CONCAT ( "/" )

    if ( stats.verComplexityLin != 0.0 ) {
        DD_CONCAT ( stats.verComplexityLin )
    } else {
        DD_CONCAT ( "-" )
    }

    DD_CONCAT( "/" )

#ifdef __STATS_OUTPUT__
    if ( Options::current().plot_required() && stats.elimComplexity > 0 ) {
        LOG_LINE( COMPLEXITY_PLOT, ( plane - mSystem.getColumns() ) << " " << stats.elimComplexity << " " << stats.verComplexityLg << " " << stats.verComplexityLin )
    }
#endif

/*	for ( unsigned long index = 0; __builtin_expect ( index < mCandidatesBuffer.size(), 1 ); index+=2 ) {

		if ( BitList::BIT_COUNT_TABLE.lookUp( p_container.sets + mCandidatesBuffer[index]*blocks , blocks ) == ( min_common_bits + 1 ) ||
			 BitList::BIT_COUNT_TABLE.lookUp( n_container.sets + mCandidatesBuffer[index+1]*blocks, blocks ) == ( min_common_bits + 1 ) ) {
			proven_faces.push_back( new SingleSetFace( p_container.rays[ mCandidatesBuffer[index] ], n_container.rays[ mCandidatesBuffer[index+1] ] ) );
			new_rays++;
		} else {		
			mRankCalculator.load ( mSystem, p_container.rays[ mCandidatesBuffer[index] ]->getActiveSet() & n_container.rays[ mCandidatesBuffer[index+1] ]->getActiveSet() );
			if ( mRankCalculator.hasMinimumRank ( min_common_bits ) ) {
				proven_faces.push_back( new SingleSetFace( p_container.rays[ mCandidatesBuffer[index] ], n_container.rays[ mCandidatesBuffer[index+1] ] ) );
				new_rays++;
			}
		}
	} */

#ifdef __COMBINATIONAL_TEST__

    mCache.pop();

	for ( unsigned long index = 0; index < mCandidatesBuffer.size(); index++ ) {
	    mCache.insert( mCandidatesBuffer[index] );
	}

#else // __ALGEBRAIC_TEST__

	for ( unsigned long index = 0; __builtin_expect ( index < mCandidatesBuffer.size(), 1 ); index+=2 ) {

		if ( mCandidatesBuffer[index]->getActiveSet().getPopulationCount() == ( min_common_bits + 1 ) ||
			 mCandidatesBuffer[index+1]->getActiveSet().getPopulationCount() == ( min_common_bits + 1 ) ) {

			proven_faces.push_back( new SingleSetFace( mCandidatesBuffer[index], mCandidatesBuffer[index+1] ) );
		} else {		
			mRankCalculator.load ( mSystem, mCandidatesBuffer[index]->getActiveSet() & mCandidatesBuffer[index+1]->getActiveSet() );
			if ( mRankCalculator.hasMinimumRank ( min_common_bits ) ) {
				proven_faces.push_back( new SingleSetFace( mCandidatesBuffer[index], mCandidatesBuffer[index+1] ) );
			}
		}
	} 

	for ( unsigned long index = 0; index < proven_faces.size(); index++ ) {
		SingleSetRay* new_ray = new SingleSetRay( proven_faces[index]->getFirst(), proven_faces[index]->getSecond(), plane, mSystem.getRows() );
		mCache.insert( new_ray, plane - mSystem.getColumns() + 1 );
		delete proven_faces[index];
	}	

#endif // _COMBINATIONAL_TEST__

	/* Clean up */
/*	for ( unsigned long index = 0; index < n_container.ray_count; index++ ) {
		delete n_container.rays[index];
	}
	
	delete[] n_container.rays;
	delete[] n_container.sets;
	delete[] n_container.starting_points;

	delete[] p_container.rays;
	delete[] p_container.sets;
	delete[] p_container.starting_points; */

	mCandidatesBuffer.clear();

	STOP_TIMER( STEP_TIME )

	DD_OUTPUT( mCache.getExtremeRays() << ", dt=" << TIMER_VALUE( STEP_TIME ) << "ms" )
}

void SingleSetRayGenerator::run () {

	for ( unsigned long plane = mSystem.getColumns(); plane < mSystem.getRows(); plane++ ) {
		calculateStep( plane );
	}
}
