/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_VANTAGE_POINT_VISITOR__
#define __DD_VANTAGE_POINT_VISITOR__

#include "DDTweaks.h"

#include "DDBitSplitVisitor.h"
#include "DDInterval.h"
#include "DDVantagePointPreliminaryCondition.h"

namespace double_description {

    class VantagePointVisitor : public BitSplitVisitor {

        protected:
        BitList mVantagePoint;

        Interval mSubDistanceRange;
        Interval mSuperDistanceRange;       

        Interval mSubPopCntRange;
        Interval mSuperPopCntRange;

        bool mInitRound;

        virtual void onSave ( const RayReference & ref ) = 0;
        virtual void onInit ( const RayReference & ref ) = 0;

        virtual bool hasFinished () const {
            return !mInitRound;
        }  

        public:
        VantagePointVisitor ( const unsigned short depth,
                              const unsigned short capacity,
                              const unsigned short split ) :
            BitSplitVisitor( depth, capacity, split ),
            mInitRound( false ) {}

        VantagePointVisitor ( const unsigned short depth,
                              const unsigned short capacity,
                              const unsigned short split, 
                              const BitList & point ) :
            BitSplitVisitor( depth, capacity, split ),
            mVantagePoint( point ),
            mInitRound( false ) {}

        virtual void onStart () {
            BitSplitVisitor::onStart();

            mSubDistanceRange.reset();
            mSuperDistanceRange.reset();

            mSubPopCntRange.reset();
            mSuperPopCntRange.reset();
        }

        void onVisit ( const struct RayReference & ref ) {
            BitSplitVisitor::onVisit( ref );             

            if ( mInitRound ) {
                onInit( ref );
            } else {
                onSave( ref );
            }
        }

        virtual void onFinish () {
            BitSplitVisitor::onFinish();
            if ( mInitRound ) {
                mInitRound = false;
            }
        }  
    };
}

#endif // __DD_VANTAGE_POINT_VISITOR__
